<?php
// $Id: ftp.backend.inc,v 1.3.2.12 2008/10/01 16:20:25 joshuarogers Exp $

/**
 * @file
 * The FTP Backend for the plugin manager.
 */

/**
 * Implementation of user hook plugin_manager_settings.  Checks
 * if the requirements for the ftp backend are enabled.
 *
 * @return
 *  Return form array of backend configuration form.
 */
function ftp_plugin_manager_settings() {
  $form = array();
  $form['plugin_manager_ftp_path'] = array(
    '#title' => t('FTP Path'),
    '#description' => t('This is the apparent path to your Drupal installation for FTP.  Leave this blank if you wish for the system to attempt to guess the correct location.<BR>(ex: If the Drupal installation is at /home/user1/www and your ftp home is /home/user1 then /www would be placed here.)'),
    '#type' => 'textfield',
    '#default_value' => variable_get('plugin_manager_ftp_path', ''),
  );

  return $form;
}

/**
 * Implementation of user hook plugin_manager_backend.  Checks
 * if the requirements for the ftp backend are enabled.
 *
 * @return
 *  Return 'ftp' if the requirements are available.
 */
function ftp_plugin_manager_backend() {
  if (function_exists('ftp_connect') OR ini_get('allow_url_fopen')) {
    return 'ftp';
  }
}

/**
 * Install the supplied files to the appropriate locations.
 *
 * @param $files
 *   The files to install.
 * @param $type
 *   Either 'module' or 'theme' to indicate where to install to.
 * @param $host
 *   FTP server (usually localhost)
 * @param $username
 *   FTP Username
 * @param $password
 *   FTP Password
 * @return
 *  TRUE if all the files were installed, FALSE otherwise.
 */
function ftp_plugin_manager_copy($files, $type, $host, $username, $password) {
  if (function_exists('ftp_connect')) {
    return ftp_plugin_manager_copy_library($files, $type, $host, $username, $password);
  }
  return ftp_plugin_manager_copy_wrapper($files, $type, $host, $username, $password);
}

/**
 * Install the supplied files to the appropriate locations using ftp without
 * the stream wrapper.
 *
 * @param $files
 *   The files to install.
 * @param $type
 *   Either 'module' or 'theme' to indicate where to install to.
 * @param $host
 *   FTP server (usually localhost)
 * @param $username
 *   FTP Username
 * @param $password
 *   FTP Password
 * @return
 *  TRUE if all the files were installed, FALSE otherwise.
 */
function ftp_plugin_manager_copy_library($files, $type, $host, $username, $password) {
  // Figure out the type of files.
  if (!in_array($type, array("modules", "themes"))) {
    drupal_set_message(t("Cannot install plugin of type @type", array('@type' => $type)), "error");
    return FALSE;
  }
  $dir = 'sites/all/'. drupal_strtolower($type);

  // Connect to the local ftp server.
  $connect = ftp_connect($host);
  if (!$connect) {
    drupal_set_message(t('No ftp server could be found.'), 'error');
    return FALSE;
  }

  // Login to the local ftp server.
  if (!@ftp_login($connect, $username, $password)) {
    drupal_set_message(t('Could not login to the ftp server.'), 'error');
    return FALSE;
  }

  // Try to guess the ftp address for the drupal install.
  // Start by putting the entire address together and replacing \ with /.
  $local_path = str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT'] . base_path());

  // Grab the last on off the end and explode it.
  $local_path = explode('/', rtrim($local_path, '/'));

  $ftp_path = variable_get('plugin_manager_ftp_path', '');

  // Try to guess which how far in we are chrooted...
  if (empty($ftp_path)) {
    foreach ($local_path AS $index => $value) {
      unset($local_path[$index]);
      if (ftp_nlist($connect, implode('/', $local_path) .'/'. $dir)) {
        $ftp_path = implode('/', $local_path) .'/'. $dir;
        drupal_set_message(t('A drupal install was automatically located on ftp at @ftp_path.', array('@ftp_path' => $ftp_path)));
        break;
      }
    }
  }
  else {
    $ftp_path = $ftp_path .'/'. $dir;
    if (!@ftp_chdir($connect, $ftp_path)) {
      drupal_set_message(t('Your provided drupal install directory is invalid. ') . l(t('Change it here.'), 'admin/plugin_manager/settings'),  'error');
      return FALSE;
    }
  }

  // If we couldn't guess it, then quit.
  if (!isset($ftp_path) || !@ftp_chdir($connect, $ftp_path)) {
    drupal_set_message(t('Could not guess the ftp directory for drupal.'), 'error');
    return FALSE;
  }

  // Prepare the directories to use later.
  $extract_dir = file_directory_path() .'/plugin_manager_extraction/';

  // Process each of the files.
  foreach ($files AS $index => $file) {
    if (trim($file, "\\/") != $file) {
      @ftp_mkdir($connect, "$ftp_path/$file");
    }
    else {
      @ftp_put($connect, "$ftp_path/$file", $extract_dir . $file, FTP_BINARY);
    }
  }
  return TRUE;
}

/**
 * Install the supplied files to the appropriate locations using the ftp
 * stream wrapper.
 *
 * @param $files
 *   The files to install.
 * @param $type
 *   Either 'module' or 'theme' to indicate where to install to.
 * @param $host
 *   FTP server (usually localhost)
 * @param $username
 *   FTP Username
 * @param $password
 *   FTP Password
 * @return
 *  TRUE if all the files were installed, FALSE otherwise.
 */
function ftp_plugin_manager_copy_wrapper($files, $type, $host, $username, $password) {
  // Figure out the type of files.
  if (!in_array($type, array('modules', 'themes'))) {
    drupal_set_message(t("Cannot install plugin of type @type", array('@type' => $type)), "error");
    return FALSE;
  }
  $dir = 'sites/all/'. drupal_strtolower($type);

  // Write the common part of the url
  $ftp_base_dir = "ftp://". urlencode($username) .":". urlencode($password) ."@$host/";

  if (!@is_dir($ftp_base_dir)) {
    drupal_set_message(t('The supplied username/password combination was not accepted, or no local ftp server is running.'), 'error');
    return FALSE;
  }

  // Try to guess the ftp address for the drupal install.
  // Start by putting the entire address together and replacing \ with /.
  $local_path = str_replace('\\', '/', $_SERVER['DOCUMENT_ROOT'] . base_path());

  // Grab the last on off the end and explode it.
  $local_path = explode('/', rtrim($local_path, '/'));

  $ftp_path = variable_get('plugin_manager_ftp_path', '');

  // Try to guess which how far in we are chrooted...
  if (empty($ftp_path)) {
    foreach ($local_path AS $index => $value) {
      unset($local_path[$index]);
      if (is_dir($ftp_base_dir . implode('/', $local_path) .'/'. $dir)) {
        $ftp_path = $ftp_base_dir . implode('/', $local_path) .'/'. $dir;
        break;
      }
    }
  }
  else {
    $ftp_path = $ftp_base_dir . $ftp_path .'/'. $dir;
  }

  // If we couldn't guess it, then quit.
  if (!isset($ftp_path) || !is_dir($ftp_path)) {
    drupal_set_message(t('Could not guess the ftp directory for drupal.'), 'error');
    return FALSE;
  }

  // Prepare the directories to use later.
  $extract_dir = file_directory_path() .'/plugin_manager_extraction/';

  // Process each of the files.
  foreach ($files AS $index => $file) {
    if (trim($file, "\\/") != $file) {
      @mkdir("$ftp_path/$file");
    }
    else {
      @copy($extract_dir . $file, "$ftp_path/$file");
    }
  }
  return TRUE;
}
